//
//	EasyMP3(VS1001K) + MMC/SD MP3 Player
//
//	2nd Version
//
//	MCU:		ATmega8		FUSE bit: D1EFh (external clock)
//	Compiler:	AVR-GCC
//
//	2003.11.18	KAWAKAMI Yukio
//


#include	<avr/io.h>
#include	<avr/signal.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#define		LF	10
#define		ON	1
#define		OFF	0
#define		TRUE	1
#define		FALSE	0

#define		SPI_CS		2	// PORTB
#define		SPI_DIN		3
#define		SPI_CLK		5
#define		SPI_DOUT	4

//#define		MP3_SO		1	// PORTB
#define		MP3_SI		7	// PORTD
#define		MP3_SCLK	6
#define		MP3_CS		5
#define		MP3_BSYNC	4
#define		MP3_DCLK	3
#define		MP3_DREQ	2

// åե饰ӥå
#define		SW_START	0
#define		SW_STOP		0
#define		SW_REW		1
#define		SW_FF		2
#define		SW_VOLDOWN	3
#define		SW_VOLUP	4

// åʪӥå
#define		SW_PORT_START	3	// PINC
#define		SW_PORT_STOP	3	// == SW_PORT_START
#define		SW_PORT_REW		2	// PINC
#define		SW_PORT_FF		1	// PINC
#define		SW_PORT_VOLDOWN	0	// PINC
#define		SW_PORT_VOLUP	0	// PINB

#define		POWER		4	// PORTC	Ÿå
#define		VOLTAGE		5	//			Ű¬
#define		MUTE		1	// PORTB	ϥߥ塼
#define		LED			1	// PORTD == TxD POWER LED

#define		BEEP_HIGH	0
#define		BEEP_LOW	1

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

// EEPROM ɥ쥹
#define		EEPROM_MUSIC		1	// ζֹ
#define		EEPROM_CLUSTER_L	2	// Υ饹(LOW)
#define		EEPROM_CLUSTER_H	3	//               (HIGH)
#define		EEPROM_REMAIN1		4	// ζʤλĤꥻ
#define		EEPROM_REMAIN2		5
#define		EEPROM_REMAIN3		6
#define		EEPROM_SIZE1		8	// ζʥǡĹ
#define		EEPROM_SIZE2		9	// SDɤ򴹤줿ɤ
#define		EEPROM_SIZE3		10	// Ƚ̤Τ˻Ȥ
#define		EEPROM_SIZE4		11
#define		EEPROM_VOLUME		12	// ܥ塼

#define		EEPROM_RUNNING_TIME_H	13	// ߤӤβƯ()
#define		EEPROM_RUNNING_TIME_M	14	//         (ʬ)
#define		EEPROM_RUNNING_TIME_OLD_H	15 // ӤβƯ()
#define		EEPROM_RUNNING_TIME_OLD_M	16 //                 (ʬ)

#define		EEPROM_BATTLOG	17	// Ű

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;

// Хåƥ꡼Ű  V * 200 - 12 (¬ˤ)
#define		BATT_START		410		// ưŰ 2.1V
#define		BATT_SHUTDOWN	380		// åȥŰ 2.0V
#define		BATT_LOW		428		// Хåƥ꡼ 2.2V
#define		BATT_26			508		// 2.6V
#define		BATT_25			488		// 2.5V
#define		BATT_24			468		// 2.4V
#define		BATT_23			448		// 2.3V
#define		BATT_22			428		// 2.2V

uchar	InitCard(void);
uchar	SPI_read_open(ulong adrs);
void	SPI_read_close(void);
uchar	SPI_command(uchar com, ulong arg);
uchar	SPI_in(void);
void	SPI_out(uchar data);
void	SPI_dummy_clock(void);
void	SPI_wait(void);
void	InitWait(void);
void	OpenWait(void);

void	read_32(ulong addr);
uchar	read_VFAT_info(void);

uint	search_MP3(uint num);
char	play_music(uint n, uint clst, ulong remainbyte);
uint	next_cluster(uint c, ulong sec, uint remain);
uint	read_word(ulong addr);
void	beep(uchar n, char err);

void	MP3_sin(PGM_VOID_P data);
void	MP3_init(void);
void	MP3_command(uchar addr, uint arg);
void	MP3_com_write(uchar data);
uint	MP3_command_read(uchar addr);
void	MP3_init(void);
void	MP3_dummy(void);

void	shutdown(char flag);
uint	check_batt(void);
void	write_eeprom(uchar adrs, uchar data);
uchar	read_eeprom(uchar adrs);
void	switch_wait(void);

uchar	ReadCom(void);
void	WriteCom(uchar data);
void	WriteComMsg(PGM_VOID_P msg);
void	WriteComStr(uchar *msg);
void	WriteHex(uchar data);

void	print_sector(ulong sec);


// ƥϢ³Ƚϥǡ		BeepȤƻȤ
PROGMEM uchar BeepData[] = { 0x53, 0xEF, 0x6E, 0x31, 0, 0, 0, 0 };	// 1500Hz
PROGMEM uchar ErrorBeep[] = { 0x53, 0xEF, 0x6E, 0x35, 0, 0, 0, 0 };	// 500Hz
PROGMEM uchar BeepStop[] = { 0x45, 0x78, 0x69, 0x74, 0, 0, 0, 0 };	// ߤ


uchar	Buff[32];
uint	Buff_p;
uint	Cluster;		// ߺΥ饹
uint	NextCluster;	// 륯饹
uint	PlayMusic;		// ζֹ
uint	MusicCount;		// ߤΥɤäƤʿ
ulong	MusicSize;		// ζʤΥ
ulong	RemainSec;		// ζʤλĤꥻ

uchar	FATtype;		// 0:FAT12  1:FAT16
uchar	SectorsPerCluster;		// 1饹Υ
uint	RootDirEntriesCount;	// 롼ȥǥ쥯ȥꥨȥ꡼
uint	FATstart;		// FATϥ
uint	DIRstart;		// 롼ȥǥ쥯ȥ곫ϥ
ulong	DataStart;		// ǡΰ賫ϥ

volatile uchar	Volume;			// 
volatile uint	BattVolt;		// ޡߤǼŸŰ
volatile uchar	SwitchFlag;		// ޡߤǼåξ

volatile uchar	StopSw;		// START/STOPåϥե饰
volatile uchar	RewSw;		// REWåϥե饰
volatile uchar	FfSw;		// FFåϥե饰
volatile uchar	VolUpSw;	// Vol upåϥե饰
volatile uchar	VolDownSw;	// Vol downåϥե饰
volatile uchar	FlushTimer;	// LEDǥ
volatile uchar	RunningTimeDS;	// Ư(1/100)
volatile uchar	RunningTimeS;	//       ()
volatile uchar	RunningTimeM;	//       (ʬ)
volatile uchar	RunningTimeH;	//       ()

ulong	FATsec;			// FATå夷Ƥ륻
uchar	FATCache[512];	// FATå

char	Debug;			// ǥХåե饰

volatile uint	BattLog[8];	// Ű


//*********************************
//	ޡСե
//	8MHz/1024/78=100.16 100Hz
//*********************************
SIGNAL(SIG_OVERFLOW0) {
//SIGNAL(SIG_OUTPUT_COMPARE2) {
	uchar	old_sw;

	// γ
	outp((uchar)(256-(78-1)), TCNT0);

	// Хåƥ꡼Фˤ
	if (!Debug){
		if (++FlushTimer >= 100){
			if (BattVolt < BATT_LOW){
				cbi(PORTD, LED);	// LED OFF
			}
			FlushTimer = 0;
		} else if (FlushTimer == 50){
			sbi(PORTD, LED);	// LED ON
		}
	}

	// ƯֵϿ
	if (++RunningTimeDS >= 100){
		RunningTimeDS = 0;
		if (++RunningTimeS >= 60){
			RunningTimeS = 0;
			if (++RunningTimeM >= 60){
				RunningTimeM = 0;
				++RunningTimeH;
			}
		}
	}

	// åå
	old_sw = SwitchFlag;
	SwitchFlag = 0;
	if (!(inp(PINC) & (1<<SW_PORT_STOP))){
		if (old_sw & (1<<SW_STOP)){
			if (++StopSw == 0)	StopSw = 255;
		} else {
			StopSw = 1;
		}
		SwitchFlag = (1<<SW_STOP);
	}

	if (!(inp(PINC) & (1<<SW_PORT_REW))){
		if (old_sw & (1<<SW_REW)){
			if (++RewSw == 0)	RewSw = 255;
		} else {
			RewSw = 1;
		}
		SwitchFlag |= (1<<SW_REW);
	}

	if (!(inp(PINC) & (1<<SW_PORT_FF))){
		if (old_sw & (1<<SW_FF)){
			if (++FfSw == 0)	FfSw = 255;
		} else {
			FfSw = 1;
		}
		SwitchFlag |= (1<<SW_FF);
	}

	if (!(inp(PINC) & (1<<SW_PORT_VOLDOWN))){
		if (++VolDownSw == 0)	VolDownSw = 255;
		if (VolDownSw >= 20){
			Volume += 8;
			if (Volume > 0xCF)	Volume = 0xCF;
			MP3_command(11, ((uint)Volume << 8)|((uint)Volume));	// 
			VolDownSw = 1;
		}
		SwitchFlag |= (1<<SW_VOLDOWN);
	} else {
		VolDownSw = 0;
	}

	if (!(inp(PINB) & (1<<SW_PORT_VOLUP))){
		if (++VolUpSw == 0)	VolUpSw = 255;
		if (VolUpSw >= 20){
			Volume -= 8;
			if (Volume < 0x1F)	Volume = 0x1F;
			MP3_command(11, ((uint)Volume << 8)|((uint)Volume));	// 
			VolUpSw = 1;
		}
		SwitchFlag |= (1<<SW_VOLUP);
	} else {
		VolUpSw = 0;
	}
	
	// A/DѴ ץꥹ 8MHz/64 = 125kHz
	outp((1<<ADEN)|(1<<ADSC)|(1<<ADIE)|6, ADCSRA);
}


//*********************************
//	A/DѴγߥ롼
//*********************************
SIGNAL(SIG_ADC) {
	uint volt;
	uchar	i;

	// A/DѴ̳Ǽ
	volt = (uint)inp(ADCL);
	BattVolt = (((uint)inp(ADCH) & 3) << 8);
	BattVolt |= volt;
	cbi(ADCSRA, ADEN);	// A/D ŸOFF

	// Ű
	for (i=7; i >= 1; --i){
		BattLog[i] = BattLog[i-1];
	}
	BattLog[0] = BattVolt;
}


//*********************************
//	INT0(EasyMP3 DREQ)ߥ롼
//*********************************
SIGNAL(SIG_INTERRUPT0) {
	// CPU򵯤
}


//*********************************
//	ᥤ롼
//*********************************
int main (void) {
	char	r;

	outp((1<<SW_PORT_VOLUP)|(1<<SPI_CS)|(1<<SPI_DIN), PORTB);	// PB0 pull up
	outp((1<<SPI_CS)|(1<<SPI_DIN)|(1<<SPI_CLK)|(1<<MUTE), DDRB);

	outp((1<<SW_PORT_STOP)|(1<<SW_PORT_REW)|(1<<SW_PORT_FF)
		 |(1<<SW_PORT_VOLDOWN), PORTC); // PC30 pull up
	outp((1<<POWER), DDRC);		// PC4 out

	outp(1, PORTD);				// RxD pull up
	outp((1<<MP3_SI)|(1<<MP3_SCLK)|(1<<MP3_CS)|(1<<MP3_BSYNC)|(1<<MP3_DCLK),
		 DDRD);

	if (inp(MCUCSR) & (1<<WDRF)){
		// åɥåꥻåȤξ
		shutdown(1);
	}
	outp(0, MCUCSR);

	if (inp(PINC)&(1<<SW_PORT_START)){
		Debug = 1;
		outp(0, UBRRH);			// DEBUG USART
		outp(24, UBRRL);		// 19200bps/8MHz
		outp(0x20, UCSRA);
		outp(0x18, UCSRB);		// 

	} else {
		Debug = 0;			// STARTåǵư
		sbi(DDRD, LED);		// LED
		sbi(PORTD, LED);	// LED ON
	}

	WDR;
	outp(0x18, WDTCR);	// WDCEWDE1ˤƥޡѹ򤹤
	outp(0x1F, WDTCR);	// åɥåޡå /3V

	outp(((1<<REFS1)|(1<<REFS0))|5, ADMUX);		// Ű 2.56V, A/D ch5

	outp((uchar)(256-(78-1)), TCNT0);	// ޡ -78
	outp(0x05, TCCR0);		// ޡץꥹ 1/1024
	sbi(TIMSK, TOIE0);		// ޳ߵ

	outp(0x83, MCUCR);		// INT0(EasyMP3 DREQ)Ω夬,꡼׵
	sbi(GICR, INT0);		// 

	SwitchFlag = 0;
	BattVolt = 0;
	FATsec = 0;

	sei();		// ߵ
	while(BattVolt < BATT_START){
		SLEEP;	// ŰͤãޤԤ
	}

	sbi(PORTC, POWER);		// Ÿå & EasyMP3 ꥻåȲ

	for (r=0; r < 35*2; r++){
		SLEEP;	// EasyMP3ΥݥåץΥᤴ
	}

	Volume = read_eeprom(EEPROM_VOLUME);
	if (Volume == 0xFF)	Volume = 0x47;

	// EEPROMƤʾ
	PlayMusic = read_eeprom(EEPROM_MUSIC);
	Cluster = (uint)read_eeprom(EEPROM_CLUSTER_H)<<8;
	Cluster |= (uint)read_eeprom(EEPROM_CLUSTER_L);
	RemainSec = (ulong)read_eeprom(EEPROM_REMAIN3)<<16;
	RemainSec |= (ulong)read_eeprom(EEPROM_REMAIN2)<<8;
	RemainSec |= (ulong)read_eeprom(EEPROM_REMAIN1);
	MusicSize = (ulong)read_eeprom(EEPROM_SIZE4)<<24;
	MusicSize |= (ulong)read_eeprom(EEPROM_SIZE3)<<16;
	MusicSize |= (ulong)read_eeprom(EEPROM_SIZE2)<<8;
	MusicSize |= (ulong)read_eeprom(EEPROM_SIZE1);

	// ƯֵϿ
	RunningTimeDS = 0;
	RunningTimeS = 0;
	RunningTimeM = read_eeprom(EEPROM_RUNNING_TIME_M);
	RunningTimeH = read_eeprom(EEPROM_RUNNING_TIME_H);
	if (RunningTimeM == 0xFF){
		RunningTimeM = 0;
		RunningTimeH = 0;
	}
	if (Debug){
		char	i;
		// Debug˲Ư֤ɽ
		WriteComMsg(PSTR(" \nRunning Time:\n"));
		WriteHex(RunningTimeH);
		WriteComMsg(PSTR("h(Hour) "));
		WriteHex(RunningTimeM);
		WriteComMsg(PSTR("h(Min)\n"));
		WriteHex(read_eeprom(EEPROM_RUNNING_TIME_OLD_H));
		WriteComMsg(PSTR("h(Hour) "));
		WriteHex(read_eeprom(EEPROM_RUNNING_TIME_OLD_M));
		WriteComMsg(PSTR("h(Min)\n"));

		// Ű
		for (i=7; i >= 0; --i){
			WriteHex(read_eeprom(EEPROM_BATTLOG+i*2));
			WriteHex(read_eeprom(EEPROM_BATTLOG+i*2+1));
			WriteComMsg(PSTR(" "));
		}
		WriteComMsg(PSTR("\n"));
	}

	// REW򲡤ʤ鵯ư鶯Ūˤζʤκǽ餫
	if (SwitchFlag & (1<<SW_REW)){
		Cluster = 0;
	}

	// MMC/SD SPI
	if (InitCard() == 0xFF){
		// MMC/SDɤ̵
		sbi(PORTB, MUTE);	// ߥ塼Ȳ
		beep(3, BEEP_LOW);
		shutdown(1);	// åȥ
	}
	sbi(PORTB, MUTE);	// ߥ塼Ȳ
	WDR;
	beep(1, BEEP_HIGH);		// BEEP

	// VFATɤ߼
	if (read_VFAT_info()){
		beep(2, BEEP_LOW);	// VFATΰ۾
		shutdown(1);
	}

	// ζʿǧ
	MusicCount = search_MP3(0xFFFF);

	if (MusicCount == 0){
		// MP3ե뤬̵
		beep(1, BEEP_LOW);
		shutdown(1);	// åȥ
	}

	if (PlayMusic == 0)	PlayMusic = 1;

	// MMC/SDɤ򴹤줿ɤĴ٤
	search_MP3(PlayMusic);
	if (MusicSize !=
		((ulong)Buff[28] + ((ulong)Buff[29] << 8)
		 + ((ulong)Buff[30] << 16) + ((ulong)Buff[31] << 24))){
		// դʤΥۤʤäƤи򴹤줿Ȥߤʤ
		PlayMusic = 1;
		Cluster = 0;
	}

	for(;;){

		if (PlayMusic > MusicCount)	PlayMusic = 1;

		switch_wait();		// åޤԤ
		// 
		r = play_music(PlayMusic, Cluster, RemainSec);
		switch_wait();		// åޤԤ
		if (r < 0){
			beep(-r, BEEP_LOW);		// 顼ȯ
			shutdown(0);
		} else if (r == 1){		// STOP줿
			if (StopSw > 100){	// 1ðʾĹϥХåƥ꡼Űå
				if (BattVolt > BATT_26){
					beep(3, BEEP_HIGH);			// 2.6Vʾ ⲻ
				} else if (BattVolt > BATT_25){
					beep(2, BEEP_HIGH);			// 2.5Vʾ ⲻ
				} else if (BattVolt > BATT_24){
					beep(1, BEEP_HIGH);			// 2.4Vʾ ⲻ
				} else if (BattVolt > BATT_23){
					beep(1, BEEP_LOW);			// 2.3Vʾ 㲻
				} else if (BattVolt > BATT_22){
					beep(2, BEEP_LOW);			// 2.2Vʾ 㲻
				} else {
					beep(3, BEEP_LOW);			// ʲ 㲻
				}
				continue;	// ³
			} else {
				shutdown(0);	// Ÿڤ
			}
		} else if (r == 3){		// REW
			if (RewSw > 100){	// 1ðʾĹϺǽζ
				PlayMusic = 1;
			} else if (RewSw > 25){	// 250msʾĹζ
				if (--PlayMusic == 0){
					PlayMusic = MusicCount;
				}
			}
			Cluster = 0;	// ʤκǽ餫
		} else {	// FF or ζʤκλ
			PlayMusic++;	// ζ
			Cluster = 0;
		}
		if (BattVolt < BATT_START){
			beep(1, BEEP_LOW);	// ưŰʲˤʤäʴ֤˷ٹ
		}
	}
}


//*********************************************************
//	åμ¤ޤԤ
//*********************************************************
void switch_wait(void){

	sei();
	for (;;) {
		WDR;
		SLEEP;
		SLEEP;
		if (!(SwitchFlag & ((1<<SW_FF)|(1<<SW_REW)|(1<<SW_STOP)))){
			SLEEP;
			SLEEP;
			if (!(SwitchFlag & ((1<<SW_FF)|(1<<SW_REW)|(1<<SW_STOP)))){
				break;
			}
		}
	}
}


//*********************************************************
//	MMC/SDɤν
//*********************************************************
uchar InitCard(void){
	uchar	c;

	outp(0x52, SPCR);	// SPIޥ, CPOL=0, å 8MHz/64=125kHz
//	outp(0x56, SPCR);	// SPIޥ, CPOL=0, CPHA = 1, å 8MHz/64=125kHz
	sbi(PORTB, SPI_CS);	// CS=H

	// MMC/SD SPI⡼ɰܹΥå
	for (c=0; c < 10; c++){
		SPI_in();
	}

	// SPI⡼ɰܹԥޥ
	cbi(PORTB, SPI_CS);		// CS=L
	c = SPI_command(0, 0);
	while ((c != 1)&&(c != 0xFF)){
		SLEEP;
		c = SPI_command(0, 0);
	}
	while (c == 1){
		c = SPI_command(1, 0);
	}
	sbi(PORTB, SPI_CS);	// CS=H

	outp(0x50, SPCR);	// SPIޥ, CPOL=0, å 8MHz/4=2MHz
//	outp(0x54, SPCR);	// SPIޥ, CPOL=0, CPHA=1, å 8MHz/4=2MHz
	sbi(SPSR, SPI2X);	// å 4MHz

	return c;
}


//*********************************************************
//	VFATɤ߼
//*********************************************************
uchar read_VFAT_info(void){
	ulong	sec;
	uint	bpb;
	uint	n;

	// VFATɤ߼ꡣĹ 512bytesǷǤƤ
	read_32(446);								// MBRɤ߹
	bpb = (uint)Buff[8] + ((uint)Buff[9]<<8);	// BPBƬ
	read_32((ulong)bpb * 512);					// ɤ߹
	n = (uint)Buff[11] + ((uint)Buff[12] << 8);	// Ĺ
	if (n != 512){
		return 2;	// 1 512bytesǤʤ
	}
	SectorsPerCluster = Buff[13];		// 饹Υ
	FATstart = bpb + (uint)Buff[14] + ((uint)Buff[15] << 8);	// ͽ󥻥
	RootDirEntriesCount = (uint)Buff[17] + ((uint)Buff[18] << 8);	// 롼ȥǥ쥯ȥꥨȥ
	n = (uint)Buff[22] + ((uint)Buff[23] << 8);	// FATΥ
	if (n == 0){
		return 1;	// FAT32餷
	}
	DIRstart = (uint)FATstart + (uint)Buff[16] * n;	// FATοߥ
	n = RootDirEntriesCount / 16;	// 롼ȥǥ쥯ȥɬפʥ
	if (RootDirEntriesCount % 16){
		n++;	// ǰΤᡢ;꤬Фڤ夲̾פȻפ
	}
	DataStart = (ulong)DIRstart + (ulong)n;		// ǡǼΰ

	sec = (ulong)Buff[19] + ((ulong)Buff[20] << 8);	// 
	if (sec == 0){
		read_32((ulong)bpb * 512 + 32);		// ɤ߹
		sec = (ulong)Buff[0] + ((ulong)Buff[1] << 8) + ((ulong)Buff[2] << 16) + ((ulong)Buff[3] << 24);
	}
	sec -= FATstart;
	sec /= SectorsPerCluster;
	if (sec >= 0x1000){
		FATtype = 1;	// FAT16
	} else {
		FATtype = 0;	// FAT12
	}

	return 0;
}


//*********************************************************
//	MP3եܤ
//	
//		num	οĤ
//		οʲǤäƤǥ쥯ȥꥨȥĴٽä
//	͡
//		Ĥ
//		Buff[]	Ǹ˸Ĥä MP3Υǥ쥯ȥ
//*********************************************************
uint search_MP3(uint num){
	uint	ent;
	ulong	sec;
	uchar	i, j;
	uint	n;

	n = 0;
	sec = (ulong)DIRstart * 512;
	for (ent = 0; ent < RootDirEntriesCount; ){
		if (SPI_read_open(sec))	return n;
		for (i = 0; i < 16; i++){
			for (j = 0; j < 32; j++){
				Buff[j] = SPI_in();
			}
			if (Buff[0] == 0)	break;	// ̤ΰ踡
			if (((Buff[26] != 0)||(Buff[27] != 0))&&	// 饹ֹ
				(Buff[0] != 0xE5)&&						// ޡ
				(!(Buff[11] & 0x18))&&					// ̾ե
				(Buff[8]=='M')&&(Buff[9]=='P')&&(Buff[10]=='3')){ // ĥMP3
				n++;
				if (--num == 0)	break;
			}
			ent++;
		}
		for (++i; i < 16; i++){
			for (j = 0; j < 32; j++){
				SPI_in();	// ʬλĤɤ߹
			}
		}
		SPI_read_close();
		if (Buff[0] == 0)	break;	// ̤ΰ踡
		if (num == 0)	break;
		sec += 512;	// Υ
	}
	return n;
}


//****************************************
//	MP3ե
//	fn:		ֹ
//	clst:	ճϥ饹==0 ξϺǽ餫
//	remainsec: ǡĤꥻ
//****************************************
char play_music(uint fn, uint clst, ulong remainsec){
	uint	i;
	ulong	sec;
	uint	remain;
	uint	cn;

	cli();		// ߶ػ
	if (fn != search_MP3(fn)){
		sei();	// ߵ
		return -1;	// ʤ̵
	}

	MusicSize = (ulong)Buff[28] + ((ulong)Buff[29]<<8) + ((ulong)Buff[30]<<16) + ((ulong)Buff[31]<<24);

	if (clst){
		// 椫
		Cluster = clst;
		RemainSec = remainsec;
		remain = MusicSize - (remainsec * 512);
	} else {
		// ǽ餫
		Cluster = (uint)Buff[26] + ((uint)Buff[27] << 8);
		RemainSec = MusicSize / 512;
		remain = MusicSize % 512;
	}

	if ((Cluster == 0xFFFF)||(Cluster < 2)){
		sei();	// ߵ
		return -2;	// 饹۾
	}

	// Υ饹Ĵ٤Ƥ
	NextCluster = next_cluster(Cluster, 0, 0);

	StopSw = 0;
	RewSw = 0;
	FfSw = 0;

	cn = SectorsPerCluster;
	sec = (((ulong)Cluster - 2) * (ulong)SectorsPerCluster + DataStart)	* 512;

	MP3_init();

	while(RemainSec > 0){
		uchar	data;
		char	bit;

		if (SPI_read_open(sec))	return -2;

		SPI_out(0xFF);		// 1byteܤΥå & WAIT

		for (i=0; i < 512; i++){

			// žԤå
			if (!(inp(PIND)&(1<<MP3_DREQ))){

				WDR;	// åɥåޡꥻå

				// Ԥ֤ˤʤä
				if (NextCluster == 0){
					// Υ饹Ĵ٤
					NextCluster = next_cluster(Cluster, sec, i);
				}

				sei();	// ߵ
				while (!(inp(PIND)&(1<<MP3_DREQ))){
					check_batt();	// Хåƥ꡼å
					SLEEP;			// ꡼
				}
				cli();	// ߶ػ
			}

			data = inp(SPDR);
			outp(0xFF, SPDR);	// Υǡꥯ

			sbi(PORTD, MP3_BSYNC);	// BSYNC = H
			for (bit=0; bit<8; bit++){
				if (data & 0x80){
					sbi(PORTD, MP3_SI);
				} else {
					cbi(PORTD, MP3_SI);
				}
				sbi(PORTD, MP3_DCLK);
				data <<= 1;
				cbi(PORTD, MP3_DCLK);
				cbi(PORTD, MP3_BSYNC);	// BSYNC = L
			}

		}
		SPI_read_close();	// ǡ 1byteʬƤ

		--RemainSec;

		if (--cn > 0){
			sec += 512;	// Υ

			if (Debug && (cn == 1)){
				uint n;
				n = BattVolt;
				WriteHex(n>>8);
				WriteHex(n);
				WriteCom(' ');
				WriteHex(Cluster>>8);
				WriteHex(Cluster);
				WriteCom('\n');
			}
		} else {
			// Υ饹
			if (NextCluster == 0){
				Cluster = next_cluster(Cluster, 0, 0);
			} else {
				Cluster = NextCluster;
				NextCluster = 0;
			}

			if ((Cluster == 0xFFFF)||(Cluster < 2)){
				// ۾說饹Ф줿ΤǶλ
				remain = 0;
				break;
			}

			sec = (((ulong)Cluster - 2) * (ulong)SectorsPerCluster
				   + DataStart) * 512;
			cn = SectorsPerCluster;

		}

		sei();	// ߵ
		check_batt();	// Хåƥ꡼å

		// STOP, FFܥå
		if (StopSw > 2){
			beep(1, BEEP_HIGH);
			return 1;
		} else if (FfSw > 2){
			beep(1, BEEP_HIGH);
			return 2;
		} else if (RewSw > 2){
			beep(1, BEEP_HIGH);
			return 3;
		}
		cli();	// ߶ػ

	}

	// (512bytes)̤ʬ
	if ((remain > 0)&&(remain < 512)){
		uchar	data;
		char	bit;

		if (SPI_read_open(sec))	return -2;
		SPI_out(0xFF);		// 1byteܤΥåСWAIT
		for (i=0; i < remain; i++){

			while(!(inp(PIND)&(1<<MP3_DREQ))){
				sei();	// ߵ
				SLEEP;	// ꡼
				check_batt();	// Хåƥ꡼å
			}
			cli();	// ߶ػ

			data = inp(SPDR);
			outp(0xFF, SPDR);	// ΥåФƤ
			sbi(PORTD, MP3_BSYNC);
			for (bit=0; bit<8; bit++){
				if (data & 0x80){
					sbi(PORTD, MP3_SI);
				} else {
					cbi(PORTD, MP3_SI);
				}
				sbi(PORTD, MP3_DCLK);
				data <<= 1;
				cbi(PORTD, MP3_DCLK);
				cbi(PORTD, MP3_BSYNC);
			}
		}
		for (; i < 512; i++){
			SPI_in();
		}
		SPI_read_close();
	}

	sei();	// ߵ

	return 0;
}


//****************************************
//	Υ饹
//****************************************
uint next_cluster(uint c, ulong sec, uint remain){
	ulong	addr;
	uint	data;
	ulong	fatadr;
	uint	i;

	if (FATtype == 0){
		// FAT12
		addr = (ulong)FATstart * 512 + (ulong)c + (c >> 1);
		if ((addr & 511) == 511){
			if (sec != 0){
				// ɤ߹ߥץʤɤ߼Τ
				for (i = remain; i < 512; i++)	SPI_in();
				SPI_read_close();

				data = read_word(addr) & 0x00FF;
				data |= ((read_word(addr+1) & 0x00FF) << 8);

				// ٥ץ󡢰ΰ֤ޤɤ߼Τ
				if (SPI_read_open(sec))	return 0;
				SPI_in();
				for (i = 0; i < remain; i++)	SPI_in();
			} else {
				// ޤ
				data = read_word(addr) & 0x00FF;
				data |= ((read_word(addr+1) & 0x00FF) << 8);
			}
		} else {
			fatadr = addr & 0xFFFFFE00;
			if ((sec != 0)&&(fatadr != FATsec)){
				// ɤ߹ߥץ FATХåե̵ʤɤ߼Τ
				for (i = remain; i < 512; i++)	SPI_in();
				SPI_read_close();

				data = read_word(addr);

				// ٥ץ󡢰ΰ֤ޤɤ߼Τ
				if (SPI_read_open(sec))	return 0xFFFF;
				SPI_in();
				for (i = 0; i < remain; i++)	SPI_in();
			} else {
				data = read_word(addr);
			}
		}
		if (c & 1){
			c = data >> 4;
		} else {
			c = data & 0x0fff;
		}
		if (c >= 0x0ff8)	c = 0xFFFF;
	} else {
		// FAT16
		addr = (ulong)FATstart * 512 + (ulong)c * 2;
		fatadr = addr & 0xFFFFFE00;
		if ((sec != 0)&&(fatadr != FATsec)){
			// ɤ߹ߥץ FATХåե̵ʤɤ߼Τ
			for (i = remain; i < 512; i++)	SPI_in();
			SPI_read_close();

			c = read_word(addr);

			// ٥ץ󡢰ΰ֤ޤɤ߼Τ
			if (SPI_read_open(sec))	return 0;
			SPI_in();
			for (i = 0; i < remain; i++)	SPI_in();
		} else {
			c = read_word(addr);
		}
		if (c >= 0xFFF8)	c = 0xFFFF;
	}

	return c;
}


//****************************************
//	MMC/SDɤ 2byteɤ
//	FATɤ߽Фˤ˻ȤΤ
//	FATCache[512]  FATsec Ȥ
//	ΥϤǤʤΤ
//	̹ǹθ뤳
//****************************************
uint read_word(ulong addr){
	uint	i;
	uint	b;
	uchar	*p;

	b = addr & 511;
	addr &= 0xFFFFFE00;

	if (addr != FATsec){
		if (SPI_read_open(addr))	return 0;
		p = FATCache;
		for (i=0; i < 512; i++){
			*p++ = SPI_in();
		}
		SPI_read_close();
		FATsec = addr;
	}

	i = (uint)FATCache[b++];
	i |= ((uint)FATCache[b] << 8);
	return i;
}


//*******************************************************
//	Хåƥ꡼ŰĴ١㤹Хåȥ󤹤
//*******************************************************
uint check_batt(void){
	uchar	i;

	if (BattVolt < BATT_SHUTDOWN){
		sbi(PORTB, SPI_CS);		// SPI CS=H
		beep(4, BEEP_LOW);
		if (RunningTimeH){
			write_eeprom(EEPROM_RUNNING_TIME_OLD_M, RunningTimeM);
			write_eeprom(EEPROM_RUNNING_TIME_OLD_H, RunningTimeH);
			RunningTimeM = 0;
			RunningTimeH = 0;
		}

		// Ű EEPROM¸
		for (i=0; i < 8; --i){
			write_eeprom(EEPROM_BATTLOG+i*2, BattLog[i]>>8);
			write_eeprom(EEPROM_BATTLOG+i*2+1, BattLog[i]);
		}

		shutdown(0);	// åȥ
	}

	return BattVolt;
}


//*********************************************************
//	åȥ
//	flag != 0 ξ EEPROM񤭹ߤԤ鷺Ÿڤ
//*********************************************************
void shutdown(char flag){

	sbi(PORTB, SPI_CS);		// SPI CS=H
	cbi(PORTB, MUTE);		// ߥ塼
	MP3_command(0, 0x0010);	// VS1001k powerdown
	if (!flag){
		write_eeprom(EEPROM_VOLUME, Volume);
		write_eeprom(EEPROM_MUSIC, PlayMusic);
		write_eeprom(EEPROM_CLUSTER_L, Cluster);
		write_eeprom(EEPROM_CLUSTER_H, Cluster>>8);
		write_eeprom(EEPROM_REMAIN1, RemainSec);
		write_eeprom(EEPROM_REMAIN2, RemainSec>>8);
		write_eeprom(EEPROM_REMAIN3, RemainSec>>16);
		write_eeprom(EEPROM_SIZE1, MusicSize);
		write_eeprom(EEPROM_SIZE2, MusicSize>>8);
		write_eeprom(EEPROM_SIZE3, MusicSize>>16);
		write_eeprom(EEPROM_SIZE4, MusicSize>>24);
		write_eeprom(EEPROM_RUNNING_TIME_M, RunningTimeM);
		write_eeprom(EEPROM_RUNNING_TIME_H, RunningTimeH);
	}
	for(;;){				// ̵¥롼
		cbi(PORTC, POWER);	// Power down
		SLEEP;
	}
}


//*******************************************************
//	EEPROM 񤭹
//*******************************************************
void write_eeprom(uchar adrs, uchar data){

	// ƱƤʤ񤭹ưԤʤ
	if (read_eeprom(adrs) != data){
		while(inp(EECR) & (1<<EEWE));
		outp(adrs >> 8, EEARH);
		outp(adrs, EEARL);
		outp(data, EEDR);
		sbi(EECR, EEMWE);
		sbi(EECR, EEWE);
	}
}


//*******************************************************
//	EEPROM ɤ߹
//*******************************************************
uchar read_eeprom(uchar adrs){

	while(inp(EECR) & (1<<EEWE));
	outp(adrs >> 8, EEARH);
	outp(adrs, EEARL);
	sbi(EECR, EERE);
	return inp(EEDR);
}


//****************************************
//	BEEP
//		n	ư
//			err	!= 0 ʤ饨顼(㤤)
//	ߵĤˤ뤳Ȥ
//****************************************
void beep(uchar n, char err){
	char	i;

	cli();	// MP3_init()˲Ĵᤵ줿麤Τǳ߶ػ
	MP3_init();
	sei();	// ߵ
	for (; n > 0; --n){
		if (err){
			MP3_sin(ErrorBeep);
		} else {
			MP3_sin(BeepData);
		}

		// Ԥδ֤ BEEPβĹˤʤ
		for (i=0; i < 10*2; i++){	// 100ms
			SLEEP;
		}

		MP3_sin(BeepStop);
	
		// BEEPβ
		for (i=0; i < 3*2; i++){	// 30ms
			SLEEP;
		}
	}

	MP3_sin(BeepStop);	// ߤޤʤȤΤǤ⤦
	
}


//****************************************
//	EasyMP3 
//****************************************
void MP3_init(void){

	MP3_command(0, 0x0004);			// Soft Reset
	InitWait();
	MP3_command(0, 0x0000);
	while(!(inp(PIND)&(1<<MP3_DREQ)));

	MP3_command(11, ((uint)Volume << 8)|((uint)Volume));	// 
	MP3_command(3, 0x8000 + (uint)((14318180)/2000));		// Clock
	InitWait();
	InitWait();
	InitWait();
}


//***********************************************************
//	EasyMP3 960bytes MP3 ե졼ʬΥߡǡ
//***********************************************************
void MP3_dummy(void){
	uint	i;
	uchar	j;

	cbi(PORTD, MP3_SI);
	for (i=0; i < 960; i++){
		while(!(inp(PIND)&(1<<MP3_DREQ)));
		sbi(PORTD, MP3_BSYNC);
		for (j=0; j < 8; j++){
			sbi(PORTD, MP3_DCLK);
			NOP;
			cbi(PORTD, MP3_DCLK);
			cbi(PORTD, MP3_BSYNC);
		}
	}
	sbi(PORTD, MP3_SI);
}


//****************************************
//	EasyMP3 ƥȽ
//****************************************
void MP3_sin(PGM_VOID_P data){
	uchar	i;
	uchar	bit;
	uchar	c;

	for (i=0; i < 8; i++){
		c = (uchar)PRG_RDB(data++);
		sbi(PORTD, MP3_BSYNC);
		for (bit=0; bit<8; bit++){
			if (c & 0x80){
				sbi(PORTD, MP3_SI);
			} else {
				cbi(PORTD, MP3_SI);
			}
			sbi(PORTD, MP3_DCLK);
			c <<= 1;
			cbi(PORTD, MP3_DCLK);
			cbi(PORTD, MP3_BSYNC);
		}
		sbi(PORTD, MP3_SI);
	}
}


//****************************************
//	EasyMP3 ޥɽ
//****************************************
void MP3_command(uchar addr, uint arg){

	cbi(PORTD, MP3_CS);
	MP3_com_write(2);
	MP3_com_write(addr);
	MP3_com_write(arg >> 8);
	MP3_com_write(arg);
	sbi(PORTD, MP3_CS);
}


//****************************************
//	EasyMP3 ޥ	̤
//****************************************
/*
uint MP3_command_read(uchar addr){
	char	i;
	uint	r = 0;

	cbi(PORTD, MP3_CS);
	MP3_com_write(3);
	MP3_com_write(addr);
	for(i=0; i<16; i++){
		sbi(PORTD, MP3_SCLK);
		r <<= 1;
		if (inp(PINB) & (1<<MP3_SO)){
			r |= 1;
		}
		cbi(PORTD, MP3_SCLK);
	}
	sbi(PORTD, MP3_CS);
	return r;
}
*/


//****************************************
//	EasyMP3 ޥ 1byte
//****************************************
void MP3_com_write(uchar data){
	char	i;

	for (i=0; i<8; i++){
		if (data & 0x80){
			sbi(PORTD, MP3_SI);
		} else {
			cbi(PORTD, MP3_SI);
		}
		sbi(PORTD, MP3_SCLK);
		data <<= 1;
		cbi(PORTD, MP3_SCLK);
	}
	sbi(PORTD, MP3_SI);
}


//****************************************
//	MMC/SDɤ 32byteɤ Buff˳Ǽ
//****************************************
void read_32(ulong addr){
	uint		i, b;

	if (SPI_read_open(addr & 0xFFFFFE00))	return;
	b = (uint)addr & 0x01FF;
	for (i=0; i < 512; i++){
		if (i == b){
			int	n;
			for (n = 0; n < 32; n++){
				Buff[n] = SPI_in();
				i++;
			}
		} else {
			SPI_in();
		}
	}
	SPI_read_close();
}


//****************************************
//	
//****************************************
void OpenWait(void){
	uint	i;

	for(i = 0; i < 10000; i++){
		NOP;
	}
}


//****************************************
//	Υå
//****************************************
void InitWait(void){
	uchar	i;

	for(i=0; i< 255; i++){
		NOP;
	}
}


//****************************************
//	MMC/SDɤ߹ߥץ
//****************************************
uchar SPI_read_open(ulong adrs){
	uint	i;
	uchar	r;
	uchar	retry;

	// ɤˤäƤϥȥ饤ɬפΤ褦
	for (retry = 0; retry < 100; retry++){
		cbi(PORTB, SPI_CS);		// SPI CS=L
		r = SPI_command(17, adrs);
		if (r == 0){
			for (i = 0; i < 2000; i++){	// ΤԤľ礬
				r = SPI_in();
				if (r == 0xFE){
					return 0;	// žϥޡ
				}
				InitWait();
			}
		} else if (r == 0xFE){
			return 0;	// 索ץ
		}
		sbi(PORTB, SPI_CS);		// SPI CS=H
		if (Debug){
			WriteHex(r);
			WriteCom(' ');
		}
		InitWait();
	}

	return 1;	// ץǤʤä
}


//****************************************
//	MMC/SDɤ߹ߥ
//	1byteΤ줬äƤⵤˤʤ
//****************************************
void SPI_read_close(void){
	uint	i;

	SPI_in();	// CRC
	SPI_in();	// CRC
	// BUSYå
	for (i = 0; i < 1000; i++){
		if (SPI_in() == 0xFF)	break;
		InitWait();
	}
	sbi(PORTB, SPI_CS);		// SPI CS=H
}


//****************************************
//	MMC/SD SPIޥɽ
//****************************************
uchar SPI_command(uchar com, ulong arg){
	uint	i;
	uchar	r;

	// BUSYå ˿ҤΥɤǤɬܤߤ
	for (i = 0; i < 1000; i++){
		r = SPI_in();
		if (r == 0xFF)	break;
		InitWait();
	}
	if (r != 0xFF)	return 1;

	SPI_out(com|0x40);
	SPI_out(arg >> 24);
	SPI_out(arg >> 16);
	SPI_out(arg >> 8);
	SPI_out(arg);
	SPI_out(0x95);		// ޥɤCRC

	for(i = 0; i < 1000; i++){
		r = SPI_in();
		if (r == 0xFE)		break;
		if (!(r & 0x80))	break;
		InitWait();
	}
	return r;
}


//****************************************
//	MMC/SD SPIǡ
//****************************************
void SPI_out(uchar data){

	outp(data, SPDR);
	while(!(inp(SPSR) & 0x80));
}


//****************************************
//	MMC/SD SPIǡ
//****************************************
uchar SPI_in(void){

	outp(0xFF, SPDR);
	while(!(inp(SPSR) & 0x80));
	return inp(SPDR);
}


//****************************************
//	MMC/SD SPIåλԤ
//	SPDR򥢥Ƥʤ
//	SPIF=0ˤʤä̵¥롼פ礬뤳Ȥ
//****************************************
void SPI_wait(void){
	while(!(inp(SPSR) & 0x80));
}	


//****************************************
//	1ɤ߹ɽ
//****************************************
void print_sector(ulong sec){
	uchar	r;
	int		i;

	sec *= 512; 
	r = SPI_read_open(sec);
	WriteHex(r);
	WriteCom('\n');
	for (i=0; i<512; i++){
		r = SPI_in();
		WriteHex(r);
		if ((i & 31) == 31){
			WriteCom('\n');
		} else if (i & 1){
			WriteCom(' ');
		}
	}
	SPI_read_close();
}


//****************************************
//	ꥢϡϤޤԤ
//****************************************
uchar	ReadCom(void){

	do {
		WDR;
	} while(!(inp(UCSRA)&(1<<RXC)));
	return inp(UDR);
}

//****************************************
//	ꥢ
//****************************************
void WriteCom(uchar data){

	do {
		;
	} while(!(inp(UCSRA)&(1<<UDRE)));
	outp(data, UDR);
}


//****************************************
//	1byte16ʿɽ
//****************************************
void WriteHex(uchar data){
	uchar	c;

	c = (data>>4) + '0';
	if (c > '9'){	c += 7; }
	WriteCom(c);

	c = (data & 0x0f) + '0';
	if (c > '9'){	c += 7; }
	WriteCom(c);
}


//****************************************
//	int 10ʸѴ
//****************************************
/*
void ItoStr(uint i, uchar str[]){
	char	*p;

	if (i >= 10000){
		p = &str[5];
	} else if (i >= 1000){
		p = &str[4];
	} else if (i >= 100){
		p = &str[3];
	} else if (i >= 10){
		p = &str[2];
	} else {
		p = &str[1];
	}
	*p-- = 0;

	do {
		*p-- = (i % 10) + '0';
		i /= 10;
	} while(i);
}
*/


//****************************************
//	ʸ
//	㡧
//		char *s;
//		s = PSTR("Message OK.\n");
//		WriteComMsg(s);
//****************************************
void WriteComMsg(PGM_VOID_P msg){
	char	i;
	uchar	c;

	i = 0;
	while((c = (uchar)PRG_RDB(msg++)) != 0){
		WriteCom(c);
		i++;
	}
}

//****************************************
//	Хåեʸ
//****************************************
void WriteComStr(uchar *msg){
	char	i;
	uchar	c;

	i = 0;
	while((c = (uchar)(*msg++)) != 0){
		WriteCom(c);
		i++;
	}
}

